from osv import osv, fields
from tools.translate import _

import time
from tools import DEFAULT_SERVER_DATETIME_FORMAT

### Shipment Promises
class shipment_promise(osv.osv):
    _name = "shipment.promise"
    _description = "Purchase Order Shipment Promise"
    _order = 'date desc'

    _columns = {
        'name': fields.char('Description', size=128),
        'purchase_order_id': fields.many2one('purchase.order', 'Purchase Order'),
        'date': fields.datetime('Contact Date'),
        'shipment_date': fields.date('Shipment Date Promised', required=True),
        'notes': fields.text('Notes'),
    }
    _defaults = {
            'date': lambda *a: time.strftime(DEFAULT_SERVER_DATETIME_FORMAT)
    }

    def _update_purchase_shipment_date(self, cr, uid, purchase_order_id, contact_date, shipment_date, this_promise_id=False, context=None):
        purchase_obj = self.pool.get('purchase.order')

        # Chequeamos si la fecha de contacto es la ultima o hay alguna posterior
        domain = [('date', '>=', contact_date), ('purchase_order_id','=',purchase_order_id)]

        # Quitamos la promise que estamos escribiendo
        if this_promise_id:
            domain.append(('id', '!=', this_promise_id))
        res = self.search(cr, uid, domain, context=context)

        # Si no hay posteriores, entonces actualizamos
        if not res:
            purchase_obj.write(cr, uid, purchase_order_id, {'date_embarked': shipment_date}, context=context)
            # Si se actualiza la promesa de envio, tambien hay que actualizar la expected date del picking asociado en caso de exsitir uno.

        return True

    def create(self, cr, uid, vals, context=None):

        purchase_order_obj = self.pool.get('purchase.order')

        # Si crean una nueva promesa de embarque...
        contact_date = vals['date']
        purchase_order_id = vals['purchase_order_id']
        shipment_date = vals['shipment_date']

        # ...tenemos que actualizar la fecha de embarque de la PO
        self._update_purchase_shipment_date(cr, uid, purchase_order_id, contact_date, shipment_date, context=context)

        return super(shipment_promise, self).create(cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        # Si modifican una nueva promesa de embarque,
        # tenemos que actualizar la fecha de embarque de la PO, solo si es necesario
        res = super(shipment_promise, self).write(cr, uid, ids, vals, context)

        # Si cambiaron la fecha de embarque o la fecha de contacto
        if 'shipment_promise' or 'date' in vals:
            # Tomamos solo una promesa de embarque, no deberian venir mas que una
            promise = self.browse(cr, uid, ids, context=context)[0]
            self._update_purchase_shipment_date(cr, uid, promise.purchase_order_id.id, promise.date, promise.shipment_date, promise.id, context)
        return res

shipment_promise()
